; Scripts for the Skype internet telephony program
; Author: Chris Nestrud <ccn@panix.com> (Skype username ChrisN)
; Contributing Developers:
; Doug Lee <dgl@dlee.org> (skype username dlee42)
; Noel Romey <ner@uark.edu> (Skype username dj_ner)
; Chad Foster
; URL: http://www.panix.com/~ccn/projects/jfw/skype.php
; Script version: 3.00
; Tested with Skype Version 1.00.097 through 1.1.0.79,  JAWS version 5.10.383 and various jaws 6 versions

include "hjConst.jsh"
include "hjGlobal.jsh"
include "common.jsm"	
include "Skype.jsh"
include "Skype.jsm"
include "SkypeCOMAPI.jsh"
include "SkypeMSAA.jsh"
include "SkypeUtil.jsl"

Function AutoStartEvent ()
if (intAutoStartRun != 1) then
SayFormattedMessage (OT_APP_START, SkypeStartMsg_L, SkypeStartMsg_S)
let cidChatIncoming = -2 ; control ID is compared in NewTextEvent. Some other control ids are 0, which caused confusion.
let intAutoStartRun = 1
EndIf
; if we re-enter a chat window, the new message code is run. force this code not to run for the first haf-second.
let intGotNewMessage=1
ScheduleFunction ("ResetGotNewMessage", 5)
EndFunction

Script HotkeyHelp ()
;Added 10/24/2004 by ner, outputs hotkey help info and home page access link.
if UserBufferIsActive () then UserBufferDeactivate() EndIf
sayFormattedMessageToUserBuffer(HotkeyHelp_MSG)
EndScript

Script SkypeScriptsHomePage ()
var string strURLString,
string strSkypeProgramDate,
string strColor,
string str
let str = GetAppTitle ()
let str = stringChopLeft (str, (StringContains (str, "-")+1))
let strSkypeProgramDate = IntToString(GetFileDate (GetAppFilePath()))
let strURLString =CNSTSkypeScriptsURL+"?JFWVersion="+IntToString(GetJFWVersion())+"&ScriptVersion="+CnstScriptVersion+"&SkypeProgramDate="+strSkypeProgramDate+"&WindowsOSVersion="+IntToString(GetWindowsOS()) + "&SkypeUserName="+str
let strColor = IniReadString ("HighlightColors", "MenuBackgroundColor", "", cnstINIFileName)
if StringLength (strColor) then
let strURLString = strURLString + "&MenuBackgroundColor=" + strColor
EndIf
let strColor = ""
let strColor = IniReadString ("HighlightColors", "ContactListBackgroundColor", "", cnstINIFileName)
if StringLength (strColor) then
let strURLString = strURLString + "&ContactListBackgroundColor=" + strColor
EndIf
let strColor = ""
let strColor = IniReadString ("HighlightColors", "CallListBackgroundColor", "", cnstINIFileName)
if StringLength (strColor) then
let strURLString = strURLString + "&CallListBackgroundColor=" + strColor
EndIf
if UserBufferIsActive () then UserBufferDeactivate() EndIf
SayFormattedMessage (OT_Message, SkypeScriptsHomePage_L, SkypeScriptsHomePage_S)
Run(strURLString)
return
EndScript

Script SayOnlineList ()
; Says who's online and on hold, or if any calls are coming in, says who that is instead.
; Author: Doug Lee
var
	int n,
	string buf,
	string list
if UserBufferIsActive () then UserBufferDeactivate() EndIf
let n = getContactList(list, "\r\n", clnAnswer)
if n > 0 then
	sayMessage(OT_Message, su_quantifyString("%% incoming call{/s}\r\n" +list, n))
	return
endIf
let n = getContactList(list, "\r\n", clnResume)
if n > 0 then
	sayMessage(OT_Message, su_quantifyString("%% call{/s} on hold\r\n" +list, n))
endIf
let n = getContactList(list, "\r\n", clnChat)
sayMessage(OT_Message, su_quantifyString("{%%/No} contact{/s} online {/\r\n}" +list, n))
EndScript

void function speakButtonGroupName()
; Speaks the group name above a set of radio buttons or check boxes.
; DGL, 07/30/04.
var
	int startType,
	int nmax
saveCursor() invisibleCursor() saveCursor()
routeInvisibleToPC()
let startType = getWindowTypeCode(getCurrentWindow())
priorLine()
let nmax = 7
while getWindowTypeCode(getCurrentWindow()) == startType && nmax
	priorLine()
	let nmax = nmax -1
endWhile
if getWindowClass(getCurrentWindow()) == "TTntTabSheet" || getWindowClass(getCurrentWindow()) == "TTntRadioGroup" then
	if !stringContains("people n|use port|ringing|keep chat hist|your contact", stringLower(stringLeft(getChunk(), 8))) then
		; The first part of that condition blocks a below-previous-field explanation in the Instant Message Alerts tab of Program Options.
		; The second keeps JAWS from reusing a "Use port" edit prompt for a subsequent check box in Program Options, Connection.
		; The third is for an unrelated prompt in the Hand/Headset tab of Options.
		; The fourth is for the "Remember my password" check box in the Privacy tab of Options.
		; The fifth is for the "Automatically pop up authorization ..." check box in Options/Chat Alerts.
		sayMessage(OT_Control_Group_Name , getChunk())
	endIf
endIf
endFunction

Void Function SpeakControl (int movement)
var
	int stc,
	int done ,
	string str 

; Shared code for handling tab, shift+tab, and JAWSKey+tab.
; Movement is 1 for tab, 2 for shift+tab, and 0 for JAWSKey+tab.
; DGL, 07/30/04.
var
	handle hwnd, handle hwnd1
if menusActive() || inHJDialog() then
	return False
endIf
If movement && UserBufferIsActive () then
	UserBufferDeActivate ()
EndIf
let hwnd = getFocus()
if movement && hwnd == getSkypeWindow(SWT_Callers) then
	let hwnd = myFindWindow(GetSkypeWindow(SWT_History), "@isGrid(%%)", "")
	if hwnd then
		speechOff()
		setFocus(hwnd)
		pause()
		speechOn()
	endIf
elif hwnd == getSkypeWindow(SWT_Main) then
	; Tab/shift+tab/ctrl+tab/etc. don't work from here, so move elsewhere, even if movement wasn't requested.
	let hwnd = getSkypeTabWindowByName(getDialogPageName())
	let hwnd1 = myFindWindow(hwnd, "@isGrid(%%)", "")
	speechOff()
	if hwnd1 then
		; This works sensibly for the Start, Contacts, and Call List tabs.
		setFocus(hwnd1)
	else
		; This works sensibly for the Dial tab and active call tabs.
		setFocus(getFirstChild(hwnd))
	endIf
	pause()
	speechOn()
elif movement == 1 then
	SpeechOff()
	{Tab}
	pause()
	SpeechOn()
elif movement == 2 then
	SayCurrentScriptKeyLabel ()
	SpeechOff()
	{Shift+Tab}
	pause()
	SpeechOn()
endIf
delay(2)
; Hook for speaking button/checkbox group names [DGL, 07/30/04].
;CF -- one function call, and eliminated double chatter with edit combos.
let stc = getWindowSubTypeCode ( getFocus () ) 
if stc == WT_EditCombo then
	if getWindowClass(getTopLevelWindow(getFocus())) == "TskSendContacts" then
		sayControlEx(getFocus(), "Send contacts to (use a comma between multiple recipients)", getObjectSubtype(), "", "", "", " ")
	else
		sayControlEx(getFocus(), getObjectName(), getObjectSubtype(), "", "", "", " ")
	endIf
	let done = True
elif stc == WT_RadioButton || stc == WT_CheckBox then
	speakButtonGroupName()
endIf
if !done then
	let done = SpeakDialogControlName(getCurrentWindow())
endIf
if !done then 
	sayObjectTypeAndText()
endIf
return True
EndFunction

string function getTabName(handle hwnd)
; Tries to get the name of the active tab, assuming the given window is a tab control.
; Author:  Doug Lee
var
	string name,
	object o, int childID
if getWindowTypeCode(hwnd) != WT_TabControl && !stringContains(stringLower(getWindowClass(hwnd)), "tabcontrol") then
	return ""
endIf
let name = getWindowName(hwnd)
if stringLength(name) then
	return name
endIf
let o = getObjectFromEvent(hwnd, 0-4, 0, childID)
if !o then
	return ""
endIf
let childID = o.accSelection
if childID == 0 then
	return ""
endIf
return o.accName(childID)
endFunction

String Function GetDialogPageName ()
; Code to make JAWS automatically announce the active page on JAWSKey+t.
; DGL, 08/01/04.
var
	handle hwnd,
	int oldRestriction,
	string result

; If JAWS already sees a page name, return that.
let result = getDialogPageName()
if result then
	return result
endIf

; Try finding the page name by searching the parents of the focus window first.
let hwnd = getFocus()
while hwnd && getWindowTypeCode(hwnd) != WT_TabControl && getWindowClass(hwnd) != "TTntPageControl"
	let hwnd = getParent(hwnd)
endWhile
let result = getTabName(hwnd)
if stringLength(result) then
	return result
endIf

; The above technique fails in the edit combo shared by all pages, so look for the visible tab sheet.
if getTopLevelWindow(getFocus()) == getSkypeWindow(SWT_Main) then
	let hwnd = getFirstChild(getSkypeWindow(SWT_PageControl))
	while hwnd
		if isWindowVisible(hwnd) && stringLength(getWindowName(hwnd)) then
			return getWindowName(hwnd)
		endIf
		let hwnd = getNextWindow(hwnd)
	endWhile
endIf

; Obsolete code that should never actually get used in the main Skype window.
saveCursor() invisibleCursor() saveCursor()
routeInvisibleToPC()
let oldRestriction = getRestriction()
setRestrictionToWindow(hwnd)
JAWSPageUp() JAWSHome()
nextLine() nextLine()  ; jumps over title and menu bar down to the tab control line.
JAWSHome()
let hwnd = getCurrentWindow()
setRestriction(oldRestriction)
let result = getTabName(hwnd)
if stringLength(result) then
	return result
endIf
return ""
EndFunction

Void Function isMultiPageDialog ()
; Code to make JAWS automatically announce the active page on JAWSKey+t.
; DGL, 08/01/04.
if isMultiPageDialog() then
	return True
endIf
return (stringLength(getDialogPageName()) > 0)
EndFunction

Void Function SwitchDocumentWindows (int isNext)
; Code to make JAWS announce the new page on ctrl+tab and ctrl+shift+tab.
var
	int tabControlActive
let tabControlActive = (getObjectTypeCode() == WT_TabControl)
if !tabControlActive then
	speechOff()
endIf
if isNext then
	performScript nextDocumentWindow()
else
	performScript previousDocumentWindow()
endIf
if !tabControlActive then
	speechOn()
	sayMessage(OT_Message, getDialogPageName() +" tab")
	; Scheduling this clears any pending schedule of this function set up by the NewTextEvent, avoiding duplicate speech.
	scheduleFunction("sayObjectTypeAndText", 1)
endIf
EndFunction

Script NextDocumentWindow ()
waitForKeysToClear(15)
switchDocumentWindows(True)
EndScript

Script PreviousDocumentWindow ()
waitForKeysToClear(15)
switchDocumentWindows(False)
EndScript

Script tab ()
if !speakControl(1) then
	performScript tab()
endIf
EndScript

Script ShiftTab ()
if !speakControl(2) then
	performScript shiftTab()
endIf
EndScript

Script SayWindowPromptAndText ()
speakControl(0)
EndScript

string function getImportantWindowText(handle hwnd)
; Get all text in hwnd up to but not including the first "stopper."
; Author:  Doug Lee
var
	string buf,
	int pos
if menusActive() || userBufferIsActive() || inHJDialog() then
	return ""
endIf
if hwnd == getTopLevelWindow(hwnd) then
	return "Top level window"
endIf
let buf = myGetWindowText(hwnd)
let buf = stopAtFirstStopper(buf, ReadingStoppers)
; Get rid of "..." on "Call With..." to shorten speech
let pos = stringContains(buf, "...")
if pos then
	let buf = stringLeft(buf, pos-1) +stringChopLeft(buf, pos+2)
endIf
return buf
endFunction

void function sayObjectTypeAndText()
; An override to improve handling of some windows.
; Author:  Doug Lee
var
	handle hwnd,
	string name,
	string buf,
	int isMainWindow
if menusActive() ||userBufferIsActive() ||dialogActive() || getCurrentWindow() == getSkypeWindow(SWT_PageControl) then
	sayObjectTypeAndText()
	return
endIf
let isMainWindow = (getTopLevelWindow(getCurrentWindow()) == getSkypeWindow(SWT_Main))
if isMainWindow then
	let hwnd = getSkypeWindow(SWT_ActiveTabSheet)
endIf
if hwnd then
	let buf = getWindowText(hwnd, Read_Everything)
else
	let hwnd = getFocus()
	let buf = ""
endIf
if isMainWindow && (stringContains(buf, "Call with...") || stringContains(buf, "Calling") || (stringContains(buf, getDialogPageName()) && getDialogPageName() != "Contact")) then
	let name = " "
elif getWindowName(hwnd) == "Conference" then
	let name = " "
elif getDialogPageName() == "Start" then
	let name = "Start page"
endIf
if stringLength(name) then
	let buf = getImportantWindowText(hwnd)
	if !buf then
		let buf = getImportantWindowText(getParent(hwnd))
	endIf
	SayControlEx (getCurrentWindow(), name, " ", " ", " ", " ", buf, " ", " ")
else
	sayObjectTypeAndText()
endIf
endFunction

Script ResetScript ()
SayFormattedMessage(OT_MESSAGE, rsStart_L)
let intAutoStartRun = 0
if (ExMessageBox ("Would you like to remove all highlight color assignments? Only answer Yes if you know what you are doing.", "Remove Highlight Color Assignments?", MB_YESNO|MB_DEFBUTTON2|MB_ICONQUESTION) == IDYES) then
if (IniRemoveSection ("Highlight Colors", "skype.jcf") == 0) then
SayFormattedMessage(OT_MESSAGE, rsRemoveNo_L)
else
IniRemoveSection ("HighlightColors", cnstINIFileName)
SayFormattedMessage(OT_MESSAGE, rsRemoveYes_L)
EndIf
EndIf
SayFormattedMessage(OT_MESSAGE, rsFinished_L)
AutoStartEvent ()
EndScript

Void Function FocusChangedEvent (handle FocusWindow, handle PrevWindow)
var
	handle RealWindow,
	string RealWindowName,
	handle AppWindow
let RealWindow = GetRealWindow (FocusWindow)
let RealWindowName = GetWindowName (RealWindow)
let AppWindow = GetAppMainWindow (FocusWindow)
if (GlobalPrevApp != AppWindow && AppWindow != FocusWindow) then
; If we've switched to a chat window, silence recognizing for a haf-second so it won't inadvertantly speak
if GetWindowClass (AppWindow) == cnstcChat then
let intGotNewMessage=1
ScheduleFunction ("ResetGotNewMessage", 5)
EndIf
endIf
focusChangedEvent(focusWindow, prevWindow)
EndFunction

Void Function NewTextEvent (handle hwnd, string buffer, int nAttributes,
int nTextColor, int nBackgroundColor, int nEcho, string sFrameName)
var string strClass,
	int stc 
let strClass = GetWindowClass (hwnd)
; take care of endlessly repeating menu text (need to localize this)
if (StringLower(buffer) == "file view tools call help") then
Return
EndIf
; Don't repeat text we've just seen
if su_stringsEqual(buffer, strTextBuffer, True) then
Return
EndIf
if isSkypeSearchWindow(hwnd) then
	; Speak text messages in the Search window (e.g., "There were no search results for ...").
	if getWindowClass(hwnd) == "TRichView" then
		sayFormattedURMessage(buffer)
	return
	endIf
	; Announce the number of results shown if it is displayed.
	if buffer == "(" && stringContains("0123456789", substring(buffer, 2, 1)) then
		sayFormattedURMessage(buffer)
		return
	endIf
endIf
if (strClass == cnstCPanel || strClass == cnstCAbout || strClass == "tSkWatermarkPanel")
		&& !stringContains(getWindowText(hwnd, Read_Everything), "Call Duration") then
;SayString(buffer)
scheduleFunction("sayObjectTypeAndText", 5)
let strTextBuffer = buffer
Return
EndIf
; only speak chat text if we're actually in the relevant chat window
if (isSkypeChatWindow(getFocus())
&& (hwnd != GetFocus ())
&& (buffer != "Last message was posted")
&& hwnd == getSkypeWindow(SWT_ChatOutput)
&& (getSkypeVar(SV_RecognizeChat) == 1)) then
if (intGotNewMessage == 0) then
SayFormattedMessage (OT_MESSAGE, NewMessage_L, NewMessage_S)
let intGotNewMessage=1
ScheduleFunction ("ResetGotNewMessage", 20)
EndIf
let strTextBuffer = buffer
Return
EndIf
let strTextBuffer = buffer
newTextEvent(hwnd, buffer, nAttributes, nTextColor, nBackgroundColor, nEcho, sFrameName)
EndFunction

Void Function sayHighlightedText (handle hwnd, string buffer)
var
	int stc
; If we're in a special situation, don't play with it.
if menusActive() || userBufferIsActive() || dialogActive() then
	sayHighlightedText(hwnd, buffer)
	return
endIf
; Don't speak highlighting outside of the current top-level window.
; Example:  When someone signs in/out and you're in a chat but the Contacts tab is also visible on screen,
; JAWS otherwise repeats the currently highlighted Contact List line.
if getTopLevelWindow(hwnd) != getTopLevelWindow(getFocus()) then
	return
endIf
if isGrid(getFocus()) then
;CF -- Double chatter with certain list boxes -- text gets painted in two strokes.
	let stc = GetWindowSubtypeCode ( hwnd )
;Only noticed with list boxes thus far.
	if stc == wt_listBox then 
;Highlighted text on same line gets painted twice.
;So, only announce first instance.
		if  stringContains ( getSelectedText () , buffer )  > 1 then 
;Second string on line, so silence.
			return
		EndIf
	EndIf
	if sayContactLine() then
		return
	endIf
endIf
sayHighlightedText(hwnd, buffer)
EndFunction

void function toolTipEvent(handle hwnd, string buffer)
; Handles tooltips as they change:
;	- Avoids extra tooltip announcements on alt+tab.
;	- Speaks a user's real name (if given) and status during arrowing in the contact list if this feature is enabled.
var
	string seltext,
	int pos, int len,
	string uname, string rname, string stat

; Filter out the speaking of tool tips when alt+tab is pressed.
if switchingTasks() then
	return
endIf

;Eliminate double chatter with tooltips -- CF
if !StringContains ( buffer , strLastTooltip ) || stringLength(buffer) != stringLength(strLastToolTip) then 
	let strLastTooltip = buffer 
	if !getContactInfo(buffer, uname, rname, stat) then
		; Added to make it easier to use ScreenEcho All to help identify toolbar controls.
		if getScreenEcho() == 2 then
			sayMessage(OT_Tool_Tip, buffer)
	endIf
		return
	endIf
	; Leave out contact name, since it is already spoken.
	; This equates to the real name in the tool tip text.
	; Resulting spoken format: [<skypeUserName> ]<statusInfo>
	;updated 09/04 by ner to include logic for turning auto speaking of tooltip on or off
	if getSkypeVar(SV_SpeakOnlineStatus) == 1 Then
		if stringLength(uname) then
			SayFormattedMessage (ot_tool_tip, uname +stat )
		else
			SayFormattedMessage (ot_tool_tip, stat )
		endIf
	EndIf
endIf
endFunction

int function tbAddGraphicButtonsWithGivenNames(string byRef names, string byRef data, int n, string givenNames)
; Adds graphic buttons to names and data.
; Assumes the JAWS or (preferably) invisible cursor is on the first button and that buttons can be reached successively via NextWord().
; Also assumes all graphics stop the cursor; use OPT_Include_Graphics=2 for best results.
; Returns the number of buttons successfully added.
; Author:  Doug Lee
var
	int i
let i = 1
while i <= n
	if !getGraphicID() then
		; TODO: Report this condition somehow.
		beep()
		return i-1  ; don't count the current "icon" that really isn't one.
	endIf
	let names = su_stringJoin(names, "|", stringSegment(givenNames, "|", i))
	; Window handle, x/y of natural JAWS/invisible cursor position for graphic, childID of 0.
	let data = su_stringJoin(data, "|",
			intToString(getCurrentWindow()) +","
			+intToString(getCursorCol()) +"," +intToString(getCursorRow()) +",0"
			)
	nextWord()
	let i = i +1
endWhile
endFunction

int function tbAddWindow(string byRef names, string byRef data, handle hwnd, string givenName)
; Add the given window, as a named toolbar item, to names and data.
; Assumes the givenName includes all type/value/state information desired; nothing is drawn from the window itself except coordinates.
; Returns 1 if successful and 0 if not.
; Author:  Doug Lee
let names = su_stringJoin(names, "|", givenName)
; Window handle, x/y of center of window, childID of 0.
let data = su_stringJoin(data, "|",
		intToString(hwnd) +","
		+intToString((getWindowLeft(hwnd)+getWindowRight(hwnd))/2) +","
		+intToString((getWindowTop(hwnd)+getWindowBottom(hwnd))/2) +",0"
		)
return 1
endFunction

int function tbAddMSAAIconsWithGivenNames(string byRef names, string byRef data, handle hwnd, string givenNames)
; Adds all MSAA toolbar items in the given window to names and data.
; Returns the number of items successfully added.
; Author:  Doug Lee
var
	object o, int childID,
	int i, int nitems, int nreal,
	string newname
let o = getObjectFromEvent(hwnd, 0-4, 0, childID)
let nitems = o.accChildCount
let nreal = 0
let i = 1
while i <= nitems
	let newname = getToolBarItemDesc(stringSegment(givenNames, "|", i), o, i, False)
	if newname then
		let names = su_stringJoin(names, "|", newname)
		; Window handle, 0,0, childID.
		let data = su_stringJoin(data, "|",
				intToString(hwnd) +",0,0,"
				+intToString(i)
				)
		let nreal = nreal +1
	endIf
	let i = i +1
endWhile
return nreal
endFunction

int function tbMakeList(string byRef names, string byRef data)
; Makes a list of buttons/controls based on the current location within Skype.
; This function is responsible for including all toolbar buttons and any special exceptions,
; such as the Call History tab's filtering buttons.
; Returns the number of items in the list.
; Author:  Doug Lee
var
	handle hwnd, handle hwnd1,
	int oldRestriction,
	int oldGMode
let names = ""
let data = ""

; First, deal with the windows that are pop-ups over the main Skype window.
let hwnd = getTopLevelWindow(getFocus())
if hwnd != getSkypeWindow(SWT_Main) then
	if getSkypeVersionCode() == 1 && isSkypeChatWindow(hwnd) then
		let hwnd = getFirstChild(hwnd)  ; the tool bar, actually with name "MainToolbar"
		tbAddMSAAIconsWithGivenNames(names, data, hwnd, TB_Chat)
	endIf
	return su_stringSegmentCount(names, "|")
endIf

; List Call History buttons and the filtering combo box if that tab is active.
if getDialogPageName() == WnHistory then
	let hwnd = getSkypeWindow(SWT_History)
	saveCursor() invisibleCursor() saveCursor()
	moveTo(getWindowLeft(hwnd)+20, getWindowTop(hwnd)+20)
	let oldGMode = getJCFOption(OPT_Include_Graphics)
	setJCFOption(OPT_Include_Graphics, 2)  ; all graphics
	let oldRestriction = getRestriction()
	setRestrictionToWindow(hwnd)
	; The four call filtering buttons left of the combo box.
	JAWSPageUp() JAWSHome()
	tbAddGraphicButtonsWithGivenNames(names, data, 4, TB_CallFilters)
	; The filter box itself.
	let hwnd1 = getSkypeWindow(SWT_Callers)
	tbAddWindow(names, data, hwnd1, stringSegment(TB_CallFilters, "|", 5) +" - " +WtComboBox +" " +getWindowText(hwnd1, Read_Everything))
	; The Clear Call List button right of the combo box.
	JAWSEnd()
	tbAddGraphicButtonsWithGivenNames(names, data, 1, stringSegment(TB_CallFilters, "|", 6))
	setJCFOption(OPT_Include_Graphics, oldGMode)
	setRestriction(oldRestriction)
	restoreCursor() restoreCursor()
endIf

; List buttons in the main toolbar if it's displayed.
; First include the two large HangUp/PickUp buttons from near the bottom of the Skype window.
; TODO: This includes window navigation code that belongs in GetSkypeWindow().
let hwnd = getSkypeWindow(SWT_MainToolBar)
if getSkypeVersionCode() >= 2 then
	; TODO: Disabled because (1) the toolbar often doesn't exist, and (2) its buttons do not correspond to the actual button locations, so clicking them causes problems.
	;tbAddMSAAIconsWithGivenNames(names, data, hwnd, TB_Main2)
else
	let hwnd1 = getFirstChild(getNextWindow(hwnd))
	tbAddMSAAIconsWithGivenNames(names, data, hwnd1, stringSegment(TB_Bottom, "|", 1))
	tbAddMSAAIconsWithGivenNames(names, data, getNextWindow(hwnd1), stringSegment(TB_Bottom, "|", 2))
	tbAddMSAAIconsWithGivenNames(names, data, hwnd, TB_Main1)
endIf
return su_stringSegmentCount(names, "|")
endFunction

int function tbClick(string data)
; Click (or otherwise activate) the button/control indicated by the given data string.
; Returns True on success and False on failure.
; Author:  Doug Lee
var
	handle hwnd, int x, int y,
	object o, int childID,
	int left, int right, int top, int bottom
let hwnd = stringToHandle(stringSegment(data, ",", 1))
let x = stringToInt(stringSegment(data, ",", 2))
let y = stringToInt(stringSegment(data, ",", 3))
let childID = stringToInt(stringSegment(data, ",", 4))
if childID then
	let o = getObjectFromEvent(hwnd, 0-4, childID, childID)
	; o.accDoDefaultAction(childID) does nothing in this app.
	if getMSAARect(o, childID, left, right, top, bottom) then
		saveCursor() JAWSCursor() saveCursor()
		moveTo( (left+right)/2, (top+bottom)/2 )
		leftMouseButtonLock()
		delay(1)
		leftMouseButtonLock()
		return True
	endIf
else
	saveCursor() JAWSCursor() saveCursor()
	moveTo(x, y)
	leftMouseButtonLock()
	delay(1)
	leftMouseButtonLock()
	return True
endIf
return False
endFunction

Script PickToolBarItem()
var
	string names, string data,
	string curcontact,
	int n, int i
let n = tbMakeList(names, data)
if !n then
	sayMessage(OT_Error, "No tool bar items found")
	return
endIf

; Translate "%c%" to the name of the current user/conference if possible.
; Skype tooltips offer this information, so why shouldn't we...
; Algorithm: Hangup submenu item if only one, else Hold submenu item if any, else "current user."
; Conference names are truncated to just the word "Conference."
; TODO:  Localize.
getContactList(curcontact, "|", clnHangup)
if stringContains(curcontact, "|") then
	getContactList(curcontact, "", clnHold)
endIf
if curcontact == "Conference" then
	let curcontact = "Conference"
endIf
if !curcontact then
	let curcontact = "current user"
endIf
su_stringReplace(names, "%c%", curcontact, "e")

let i = dlgSelectItemInList(names, "Select a ToolBar Item", False)
speechOff()
stopSpeech()
pause()
speechOn()
if !i then
	sayMessage(OT_Status, "Canceled")
else
	tbClick(stringSegment(data, "|", i))
endIf
pause()
scheduleFunction("sayObjectTypeAndText", 3)
EndScript

String Function ToggleRecognizeChat (int RetCurValue)
return skypeVarCycler(SV_RecognizeChat, msgDisabledEnabled, RetCurValue)
EndFunction

String Function ToggleSpeakOnlineStatus (int RetCurValue)
return skypeVarCycler(SV_SpeakOnlineStatus, msgDisabledEnabled, RetCurValue)
EndFunction

String Function ToggleSkypeWatchActive (int RetCurValue)
return skypeVarCycler(SV_SkypeWatchActive, msgDisabledEnabled, RetCurValue)
EndFunction

Script ReadChatErrorMessages()
var
	handle hwnd,
	string buf
let hwnd = getSkypeWindow(SWT_ChatErrors)
if !hwnd || !isWindowVisible(hwnd) then
	sayMessage(OT_Error, NoChatErrors)
	return
endIf
let buf = getWindowText(hwnd, Read_Everything)
if stringIsBlank(buf) then
	sayMessage(OT_Error, NoChatErrors)
	return
endIf
sayFormattedURMessage(buf)
EndScript

Script GoToResponsesList ()
if UserBufferIsActive () then UserBufferDeactivate() EndIf
if (!hndFSearchResponses) then
let hndFSearchResponses = GetFirstChild(GetFirstChild(GetAppMainWindow(GetCurrentWindow())))
EndIf
if (GetWindowClass (GetAppMainWindow(GetCurrentWindow())) != cnstCSearch) then
SayFormattedMessage(OT_Error, NotInSearchDialog_L, NotInSearchDialog_S)
Return
EndIf
if (GetCurrentWindow () == hndFSearchResponses) then
SayFormattedMessage(OT_Error, AlreadyInResponses_L, AlreadyInResponses_S)
return
EndIf
SetFocus(hndFSearchResponses)
EndScript

Script ScriptFileName ()
ScriptAndAppNames ("skype")
SayFormattedURMessage(SkypeAppVersionStringMsg_L)
EndScript

int function sayContactLine()
;Updated CF -- Aug 15, 2004
;Call list page has additional info about history that is captured in the selected text.
var
	handle hwnd,
	string grname,
	string line,
	string sel

if !isPCCursor() || menusActive() || userBufferIsActive() || inHJDialog() || switchingTasks() then
	return False
endIf
;Compare two values -- selected text and line.
let line = getLine ()
; Remove "scroll up symbol" because it otherwise speaks on the top contact line [DGL, 2004-10-03].
if stringRight(line, 16) == "scroll up symbol" then
	let line = stringTrimTrailingBlanks(stringChopRight(line, 16))
endIf
let sel = getSelectedText () 
if StringLength ( line ) < stringLength ( getSelectedText () ) then 
;Speak the longer value.
	Say ( sel ,  ot_line )
else 
	Say ( line ,  ot_line )
endIf
if getActiveCursor() != Cursor_JAWS then
	saveCursor() JAWSCursor()
	routeJAWSToPC()
	nextCharacter()
endIf
return True
endFunction

Script SayLine ()
; Makes SayLine work in TKGrid and TTntDrawGrid controls, where it is otherwise silent.
; The isSameScript part lets the normal SpellLine behavior work.
; DGL, 07/09/04.
if !isSameScript() && isPCCursor() && !menusActive() && !userBufferIsActive() && isGrid(getFocus()) then
	if !sayContactLine() then
		sayLine()
	endIf
	return
endIf
performScript sayLine()
EndScript

String Function GetInternalControlName (handle hnd)
var handle hndCurrent,
handle hndLast,
handle hndAppMainWindow,
string strInternalName,
string strDialog
let hndAppMainWindow = GetAppMainWindow(GetCurrentWindow())
let hndCurrent = hnd
while (hndCurrent != hndAppMainWindow)
let hndLast = hndCurrent
let hndCurrent = GetParent (hndLast)
EndWhile
let strInternalName = StringSegment (GetWindowClass(hndAppMainWindow), ".", 1) + ".W" + IntToString(GetWindowHierarchyX (hndLast)) + ".X" + IntToString(GetWindowHierarchyX (hnd)) + ".Y" + IntToString(GetWindowHierarchyY (hnd))
if (IsMultiPageDialog ()) then
;SayString("multipage")
let strInternalName = strInternalName + ".D" + GetDialogPageName ()
EndIf
; let strInternalName = strInternalName + "." + GetWindowPath(hnd)
return strInternalName
EndFunction

String Function GetWindowPath (handle hnd)
var handle hndAppMainWindow,
string strPath
let hndAppMainWindow = GetAppMainWindow(hnd)
while (hnd != hndAppMainWindow)
if (GetWindowHierarchyX (hnd) == 1) then
let hnd = GetParent (hnd)
let strPath = "f" + strPath
else
let hnd = GetPriorWindow(hnd)
let strPath = "n" + strPath
EndIf
EndWhile
return "f" + strPath
EndFunction

int function switchToSkypeTab(variant which)
; Switches to the indicated Skype main-window tab or announces that it is already active.
; Pressing any command that calls this function twice quickly will instead place focus on the tab control.
var
	handle hwnd,
	object o, int childID
if isSameScript() then
	let hwnd = getSkypeWindow(SWT_PageControl)
	if !hwnd then
		sayFormattedMessage(OT_Error, TabNotFound_L, TabNotFound_S)
		return False
	endIf
	if getFocus() == hwnd then
		sayObjectTypeAndText()
	else
		setFocus(hwnd)
	endIf
	return True
endIf

let o = getSkypeMSAAObject(SWT_PageControl)
if !o then
	return False
endIf
if stringToInt(which) then
	let childID = stringToInt(which)
	; We can't use o.accChildCount to count tabs, because the actual tab sheet windows are also considered children of the page control object.
	; Requesting the role of a child ID with no corresponding tab will return 0.
	if !o.accRole(childID) then
		sayFormattedMessage(OT_Error, NotEnoughTabs_l, NotEnoughTabs_S)
		return False
	endIf
else
	let childID = findMSAAChildByName(o, which)
	if !childID then
		sayMessage(OT_Error, formatString(MoveTabNotFound_l, which))
		return False
	endIf
endIf
if IsSkypeTabActive (ChildID, o) then
	sayMessage(OT_Error, formatString(AlreadyActive_l, getDialogPageName()))
	sayObjectTypeAndText()
	return True
endIf
o.accDoDefaultAction(childID)
pause()
sayMessage(OT_Message, getDialogPageName() + " " + TabMsg_s)
return True
endFunction

script switchToTab()
var
	string which
waitForKeysToClear(15)
let which = stringRight(getCurrentScriptKeyName(), 1)
switchToSkypeTab(which)
endScript

Script SayCurrentTabControl ()
;added by ner 09/02/04
;this script at the moment speaks the current tab only, a quick  to speak the tab rather than just pressing insert t.
;Function is ambiguously named at the moment, but ultimately, I'd like to have a status script of some kind to be named the current script name
if UserBufferIsActive () then UserBufferDeactivate() EndIf
SayFormattedURMessage (FormatString (YouAreInWhichTab_l, GetDialogPageName()) +"|" +GetDialogPageName())
EndScript





string function positionInGroup()
; Override of standard JAWS function to make the current position in some combo boxes speak.
; Author:  Doug Lee
var
	handle hwnd,
	string result,
	object o, int childID,
	int i, int n
let result = positionInGroup()
if result then
	return result
endIf
let hwnd = getCurrentWindow()
if getWindowTypeCode(hwnd) == WT_ComboBox || stringContains(stringLower(getWindowClass(hwnd)), "combo") then
	; Window.client.child3 (which is the list window) .client (which is the actual list object)
	let o = getObjectFromEvent(hwnd, 0-4, 0, childID).accChild(3).accChild(0-4)
	if !o then
		return ""
	endIf
	let i = o.accSelection
	let n = o.accChildCount
	if i then
		return intToString(i) +" of " +intToString(n)
	else
		return intToString(n) +" items"
	endIf
endIf
return ""
endFunction

Function ResetGotNewMessage ()
let intGotNewMessage=0
EndFunction

int function saveHighlightColor(string clrdesc, string searchstr, int n, int reportIfFound)
; Saves the highlight color found on line n of the current window.
; Helper for the CustomHighlightTrainer script below.
; Author:  Doug Lee
var
	int oldRestriction,
	int clr,
	string sclr
saveCursor() invisibleCursor() saveCursor()
routeInvisibleToPC()  ; right window, probably wrong location if highlight tracking doesn't work yet
let oldRestriction = getRestriction()
setRestriction(RestrictWindow)
JAWSPageUp()
JAWSHome()
if stringLength(getCharacter()) > 1 then  ; e.g., "Scroll up symbol"
	nextLine()
	JAWSHome()
endIf
if !stringContains(getLine(), "/") then  ; e.g., "Skype Contacts (5/20)"
	; TODO:  Report this.
	return False
endIf
while n > 1
	nextLine()
	JAWSHome()
	let n = n -1
endWhile
let clr = getColorBackground()
let sclr = "AnyColor/" +colorToRGBString(clr)
setRestriction(oldRestriction)
if stringLength(searchStr) then
	if !stringContains(stringLower(getLine()), stringLower(searchStr)) then
		return False
	endIf
endIf
if (IniReadInteger ("Highlight Colors", sclr, 0, "skype.jcf") == 1) then
	if reportIfFound then
		sayFormattedURMessage(clrdesc +":  " +htAlreadyRecognized_L +":  " +getColorName(clr) +", " +sclr)
	endIf
	Return True
EndIf
if (!IniWriteInteger ("Highlight Colors", sclr, 1, "skype.jcf")) then
	sayFormattedURMessage(htWriteFailed_L)
	Return False
EndIf
sayFormattedURMessage(clrdesc +" color added")
return True
endFunction

Script CustomHighlightTrainer ()
; Gets color at pc cursor location if in a menu, contact list, or call list. Saves highlight color in skype.jcf to be used in cursor tracking.
Var String BGColor,
string CHighlightString,
int intIniStatus,
string strDPN ; dialog page name
waitForKeysToClear(15)
SayFormattedMessage(OT_Message, htStart_L, htStart_S)
; DGL: Special treatment of the new TreeView-like Contact List.
if getSkypeVersionCode() == 2 && getParent(getFocus()) == getSkypeWindow(SWT_Contacts) && isGrid(getFocus()) then
	speechOff()
	PCCursor()
	JAWSHome()  ; sends a Home key which goes to the top of the list
	pause()
	priorCharacter()  ; closes the contact list, which changes the highlight color of the top line
	pause()
	nextLine()  ; Goes to the next main tree section, if any
	pause()
	saveHighlightColor("Closed SkypeOut contact group", "SkypeOut", 2, False)
	nextCharacter()
	saveHighlightColor("Open SkypeOut contact group", "SkypeOut", 2, False)
	JAWSHome()  ; Returns to the top of the tree, which is still closed
	saveHighlightColor("Closed contact group", "/", 1, True)
	nextCharacter()  ; opens the contact list
	pause()
	saveHighlightColor("Open contact group", "/", 1, True)
	nextLine()  ; goes to the first entry in the list
	pause()
	saveHighlightColor("Selected contact", "", 2, True)
	speechOn()
	return
endIf

; Where are we?
if MenusActive() == MENU_INACTIVE then
let strDPN = GetDialogPageName ()
EndIf
if ((!StringLength(STRDPN) && MenusActive () == MENU_INACTIVE) || (StringLength(strDPN) && strDPN != "Contacts" && strDPN != "Call")) then
; We aren't in a trainable location.
SayFormattedMessage(OT_MESSAGE, htWhereAmI_L)
Return
EndIf
; set proper position if we're in a supported tab
if (strDPN == "Contacts" || strDPN == "Call") then
; try to move to list if we aren't in one
if !IsGrid(GetFocus()) Then
SayFormattedMessage(OT_MESSAGE, htMove_L)
SpeechOff()
{tab}
delay(5)
SpeechOn()
EndIf
; ...if we still aren't in a list
if !IsGrid(GetFocus()) then
SayFormattedMessage(OT_MESSAGE, htMoveFail_L)
Return
EndIf
EndIf ; on a tab
; position is fine if we're in a menu
; we are now in a trainable location
Let BGColor = GetSkypeHighlight()
;If on a grid, makes sure that something is actually highlighted and that Jaws is on the first item of a list
if (isGrid(GetFocus()) && (bgColor == "255255255" || bgColor == "000000000")) then
;If the background is white or black, first click the mouse to select a highlight, and then reget the color
SpeechOff()
LeftMouseButton()
delay(7)
Let BGColor = GetSkypeHighlight()
;display error if the clicking didn't work
SpeechOn()
if (bgColor == "255255255" || bgColor == "000000000") then
SayFormattedMessage(OT_MESSAGE, htBadColor_L)
SayString(bgColor)
return
EndIf ; show error
EndIf ; click
Let CHighlightString = "AnyColor" + "/" + BGColor
if (IniReadInteger ("Highlight Colors", cHighlightString, 0, "skype.jcf") == 1) then
SayFormattedMessage(OT_MESSAGE, htAlreadyRecognized_L)
SayString(bgColor)
Return
EndIf
if (!IniWriteInteger ("Highlight Colors", CHighlightString, 1, "skype.jcf")) then
SayFormattedMessage(OT_MESSAGE, htWriteFailed_L)
Return
EndIf
; Keep track of color for possible reporting later
if !intIniStatus && MenusActive() != MENU_INACTIVE then
let intIniStatus = IniWriteString ("HighlightColors", "MenuBackgroundColor", bgColor, cnstINIFileName)
EndIf
if !intIniStatus && strDPN == "Contacts" then
let intIniStatus = IniWriteString ("HighlightColors", "ContactListBackgroundColor", bgColor, cnstINIFileName)
EndIf
if !intIniStatus && strDPN == "Call" then
let intIniStatus = IniWriteString ("HighlightColors", "CallListBackgroundColor", bgColor, cnstINIFileName)
EndIf
if (!intIniStatus) then
SayString("ini write failed")
EndIf
; Everything is finished.
SayFormattedMessage(OT_MESSAGE, htRecognized_L)
EndScript

String Function GetSkypeHighlight ()
; returns background color at pc cursor location
Var string BGColor
SaveCursor()
SpeechOff()
RouteJawsToPC()
Delay(2) 
if !getCharacter() then
	JAWSHome()
	pause()
endIf
Let BGColor=ColorToRGBString(GetColorBackground ())
RestoreCursor()
SpeechOn()
Return BGColor
EndFunction

Void Function SpeakDialogControlName (handle hwnd)
var
	string strKey,
	string name
if stringContains(stringLower(getWindowClass(getParent(hwnd))), "datetime") then
	let name = "Birthdate"
elif stringContains(stringLower(getWindowClass(getTopLevelWindow(hwnd))), "search") && isGrid(hwnd) then
	let name = "Responses"
endIf
if !name then
	let strKey = GetInternalControlName (hwnd)
	let name = IniReadString ("DialogControls", strKey, "", "skype.ini")
	;if name then sayString("Ini") endIf
endIf
if name then
	SayControlEx (hwnd, name, getObjectType(), getObjectState(), "", "", getObjectValue(), positionInGroup(), "")	
	return True
endIf
return False
EndFunction

int function dialogActive()
; Override to make the result more accurate for Skype.
; Author:  Doug Lee
var
	handle hwnd,
	string wclass
if dialogActive() then
	return True
elif menusActive() || userBufferIsActive() then
	return False
endIf
let hwnd = getTopLevelWindow(getFocus())
let wclass = getWindowClass(hwnd)
if stringRight(wclass, 13) == ".UnicodeClass" then
	let wclass = stringChopRight(wclass, 13)
endIf
if wclass == "Tsk" && stringRight(wclass, 4) == "Form" && wclass != "TskMainForm" then
	return True
endIf
return False
endFunction

script Enter()
var
	handle hwnd, handle hwnd1
let hwnd = getFocus()
typeCurrentScriptKey()
if !dialogActive() then
	return
endIf
pause()
if hwnd != getFocus() then
	return
endIf
let hwnd = getTopLevelWindow(hwnd)
let hwnd1 = findWindow(hwnd, "", "ok")
if !hwnd1 then
	let hwnd1 = findWindow(hwnd, "", "Save")
endIf
if !hwnd1 then
	let hwnd1 = findWindow(hwnd, "", "Search")
endIf
if !hwnd1 then
	let hwnd1 = findWindow(hwnd, "", "Next")
endIf
if !hwnd1 then
	let hwnd1 = findWindow(hwnd, "", "Finish")
endIf
if !hwnd1 then
	let hwnd1 = findWindow(hwnd, "", "Search")
endIf
if hwnd1 then
	saveCursor() JAWSCursor() saveCursor()
	moveTo( (getWindowLeft(hwnd1)+getWindowRight(hwnd1))/2, (getWindowTop(hwnd1)+getWindowBottom(hwnd1))/2)
	leftMouseButton()
endIf
endScript

void function keyPressedEvent(int nKey, string strKeyName, int nIsBrailleKey, int nIsScriptKey)
; Makes the Applications (Context Menu) key work where it should but doesn't already.
; Author:  Doug Lee
keyPressedEvent(nKey, strKeyName, nIsBrailleKey, nIsScriptKey)
if nKey == 93 then
	scheduleFunction("checkForMenu", 3)
endIf
endFunction

void function checkForMenu()
; Helper for KeyPressedEvent.
if menusActive() then
	return
endIf
saveCursor() JAWSCursor() saveCursor()
routeJAWSToPC()  ; the PC cursor may not be active, but the Applications key always works with the focus, so we will too.
rightMouseButton()
endFunction

int function findNthColorTransition(handle hwnd, int delta, int n)
; Finds the nth color transition in delta's direction from the current active cursor's location.
; Returns True on success and False on failure.
; The active cursor is moved to the found spot only if a spot is found.
; The search is limited by the edge of hwnd in delta's direction if hwnd is not null.
; Author:  Doug Lee
var
	int x, int y,
	int limit, int tooFar,
	int clr, int clr0
if hwnd then
	if delta > 0 then
		let limit = getWindowRight(hwnd)
	else
		let limit = getWindowLeft(hwnd)
	endIf
else
	if delta > 0 then
		let limit = screenGetWidth() +1
	else
		let limit = 0
	endIf
endIf
let x = getCursorCol() +(delta *3)  ; arbitrary safety margin to avoid edge of text characters
let y = getCursorRow()
let clr0 = getColorAtPoint(x, y)
let tooFar = False
while n && !tooFar
	let clr = getColorAtPoint(x, y)
	if clr != clr0 then
		let n = n -1
		let clr0 = clr
	endIf
	let x = x +delta
	if delta > 1 then
		let tooFar = (x > limit)
	else
		let tooFar = (x < limit)
	endIf
endWhile
if tooFar then
	return False
endIf
moveTo(x, y)
return True
endFunction

int function clickMoreResultsGraphic()
; Clicks the plus sign in the Search dialog used to increase the number of search results shown.
; Also avoids the focus shift that otherwise accompanies this action, because that shift disables arrow keys in the list.
; Returns True on success and False on failure.
; Author:  Doug Lee
var
	handle hwnd
let hwnd = getTopLevelWindow(getFocus())
if !isSkypeSearchWindow(hwnd) then
	sayMessage(OT_Error, "Not in the Search window")
	return False
endIf
saveCursor() invisibleCursor() saveCursor()
moveTo(getWindowLeft(hwnd)+3, getWindowTop(hwnd)+3)
findString(hwnd, "show more results", S_Bottom, S_Unrestricted)
if getChunk() != "Show more results" then
	sayMessage(OT_Error, "There are no more results to show")
	return False
endIf
if !findNthColorTransition(getCurrentWindow(), -1, 6) then
	sayMessage(OT_Error, "Unable to find the plus sign to click")
	return False
endIf
let hwnd = getFocus()
speechOff()
saveCursor() JAWSCursor()
routeJAWSToInvisible()
leftMouseButton()
pause()
if getFocus() != hwnd then
	setFocus(hwnd)
	pause()
endIf
speechOn()
return True
endFunction

int function sayResultCount()
; Says the number of search results shown, if that number is shown on screen.
var
	handle hwnd,
	string buf
let hwnd = getTopLevelWindow(getFocus())
if !isSkypeSearchWindow(hwnd) then
	sayMessage(OT_Error, "Not in the Search window")
	return False
endIf
let buf = stringTrimTrailingBlanks(getWindowText(hwnd, Read_Everything))
if stringRight(buf, 1) != ")" then
	return False
endIf
let buf = stringChopLeft(buf, stringContainsFromRight(buf, "("))
let buf = stringChopRight(buf, 1)
sayFormattedURMessage(buf)
return True
endFunction

script GetMoreSearchResults()
if isSkypeSearchWindow(getFocus()) then
	clickMoreResultsGraphic()
	return
endIf
sayCurrentScriptKeyLabel()
typeCurrentScriptKey()
endScript

Int Function CountActiveSkypeTabs ()
; Returns the number of available tabs in the main Skype window.
; Author: ner
var
	object o,
	int counter
let o = getSkypeMSAAObject(SWT_PageControl)
if !o then
	return False
endIf
Let Counter = 1
While O.AccRole(Counter)
	let Counter = Counter+1
EndWhile
let counter = counter-1  ; the counter ended up pointing at the first invalid tab, not the last valid one.
return counter
EndFunction

void function sayTabName()
sayFormattedURMessage(getDialogPageName())
endFunction

Script GetSkypeActiveTabInfo ()
var
	object o,
	int ChildID,
	int makingList,
	String tabname,
	String buf,
	int numtabs
WaitForKeysToClear(15)
let makingList = isSameScript()
let numtabs = CountActiveSkypeTabs ()
;get number of tabs available and then say how many active tabs there are
if !makingList then
	SayFormattedURMessage(FormatString(TabCountMsg_L, IntToString(NumTabs)) +"|" +FormatString(TabCountMsg_S, IntToString(NumTabs)))
endIf
let o = getSkypeMSAAObject(SWT_PageControl)
if !o then
	;	return False
endIf
let childID = 1
While childID <= numtabs
	let tabname = o.accName(childID)
	if buf then
		if makingList then
		let buf = buf +"|"
		else
			let buf = buf +"\r\n"
		endIf
	endIf
	if ChildID == numtabs && !makingList && ((GetVerbosity() == beginner) || (GetVerbosity() == intermediate)) then
		; Make it gramatically correct and insert "and" into the list.
		let tabname = and_MSG+" "+Tabname
		; If Verbosity is advanced, to lower chatter, do not speak And
	endIf
	if !makingList && IsSkypeTabActive (ChildID, o) then
		if GetVerbosity() == (beginner || intermediate) then
			let buf = buf +formatString(BufActiveTab_l, tabname)
		else
			let buf = buf +formatString(BufActiveTab_s, tabname)
		EndIf
	else
 		let buf = buf +tabname
	EndIf
	let childID = childID +1
EndWhile
if !makingList then
	sayFormattedURMessage(buf)
	return
endIf
let childID = dlgSelectItemInList(buf, TabListTitle, False)
speechOff()
stopSpeech()
pause()
speechOn()
if !childID then
	sayMessage(OT_Status, "Canceled")
else
	o.accDoDefaultAction(childID)
endIf
pause()
scheduleFunction("sayTabName", 3)
EndScript

Int Function IsSkypeTabActive (int ChildID, object o)

if o.accState(childID) & 2 then  ; 2 = State_System_Selected
return true
else
return false
EndIf

EndFunction

Script AdjustJawsVerbosityInSkype ()
; Uses Jaws's central verbosity coding.
; Modified for skype usage by Ner
; To write more toggle functions for this, see toggle functions
; They *must* be done in the same manner.

if IsSkypewatchLoaded() then
; If SkypeWatch is loaded then add verbosity strings
JAWSVerbosityCore (CNSTSkypeToggleList +"|" +CNSTSkypeWatchToggleList )
else
JAWSVerbosityCore (CNSTSkypeToggleList)
EndIf

EndScript

script sayBottomLineOfWindow()
; Helps when, for example, an open chat window confuses the normal JAWSKey+PageDown command even though focus is in the main Skype window.
var
	handle hwnd
let hwnd = findWindowByTypeCode(getTopLevelWindow(getFocus()), WT_StatusBar, False)
if hwnd && isWindowVisible(hwnd) && !isWindowObscured(hwnd) then
sayFormattedURMessage(getWindowText(hwnd, Read_Everything))
	return
endIf
performScript sayBottomLineOfWindow()
endScript
