; Skype scripts that can run even when Skype is not visible.
;
; Author:  Doug Lee of Bartimaeus Group

include "hjconst.jsh"
include "skype.jsh"
include "skype.jsm"
include "SkypeCOMAPI.jsh"
include "SkypeMSAA.jsh"
include "SkypeUtil.jsl"

const
; Values for sw_state (the state of Skype itself)
	SWS_Off = 0,
	SWS_On = 1

globals
; The state of Skype itself.
	int sw_state,
; The online/offline/etc. state of the current Skype user.
	int sw_status,
; Lists of contacts.
	string sw_onLine, string sw_active, string sw_onHold, string sw_calling

void function autoStartEvent()
scheduleFunction("skypeWatcher", 20)
endFunction

void function sw_say(string s)
if getSkypeVar(SV_SkypeWatchActive) then
	sayFormattedURMessage(s)
endIf
endFunction

string function sw_getMissingElements(string s1, string s2, string sep, string prefix)
var
	string changelist,
	string seg
let s2 = sep+s2+sep
while s1
	let seg = su_stringSegmentNext(s1, sep)
	if !stringContains(s2, sep+seg+sep) then
		let changelist = su_stringJoin(changelist, sep, prefix+seg)
	endIf
endWhile
return changelist
endFunction

string function sw_getChangeList(string s1, string s2, string sep)
; Returns a list of changes between s1 and s2, which should be segmented with sep.
; Additions to s2 not in s1 are preceded by a plus, and removals by a dash in the returned string.
; The returned string is delimited by sep also, and removals precede additions.
; The results are not sorted, and the input strings need not be either.
var
	string changelist
let changelist = sw_getMissingElements(s1, s2, sep, "-")
let changelist = su_stringJoin(changelist, sep, sw_getMissingElements(s2, s1, sep, "+"))
return changelist
endFunction

int function sw_checkForChanges(string which, string byRef lastSnapshot, string onAdd, string onRemove)
var
	string curSnapshot,
	string seg,
	string changelist,
	int isAdd,
	int safetyCount
if !getContactList(curSnapshot, "|", which) then
	let curSnapshot = ""
endIf
let changelist = sw_getChangeList(lastSnapshot, curSnapshot, "|")
let lastSnapshot = curSnapshot +""
if !stringLength(changeList) then
	return False
endIf
while stringLength(changelist)
	let seg = su_stringSegmentNext(changelist, "|")
	let isAdd = (stringLeft(seg, 1) == "+")
	let seg = stringChopLeft(seg, 1)
	if isAdd then
		sw_say(formatString(onAdd, seg))
	else
		sw_say(formatString(onRemove, seg))
	endIf
endWhile
return True
endFunction

void function skypeWatchHelper()
; Does the work of SkypeWatcher().
var
	int newstat,
	int changed

; If neither speech nor Braille are in use, do nothing because announcements wouldn't happen anyway.
; This can happen, for example, when Braille and speech are disabled in a DOS box.
if !speechInUse() && !BrailleInUse() then
	return
endIf

; What if Skype isn't running...
if !getSkypeWindow(SWT_Main) then
	if sw_state != SWS_Off then
		sw_say("Skype unloaded")
		let sw_state = SWS_Off
		let sw_status = -1
		let sw_onLine = ""
		let sw_active = ""
		let sw_onHold = ""
		let sw_calling = ""
	endIf
	return
endIf
let sw_state = SWS_On

; Check the current user's status for changes.
let newstat = getMyOnlineStatus()
if newstat != sw_status then
	sw_say("Skype status "
			+stringSegment("unknown|offline|online|away|not available|invisible|do not disturb|Skype me", "|", newstat+2))
	let sw_status = newstat
endIf

; If the user is offline, don't look for changes in contact lists.
if newstat == OLSOffline then
	return
endIf

let changed = sw_checkForChanges(clnAnswer, sw_calling, "Incoming call from %1", "")
let changed = sw_checkForChanges(clnHold, sw_active, "%1 call active", "") || changed
let changed = sw_checkForChanges(clnResume, sw_onHold, "%1 placed on hold", "") || changed
let changed = sw_checkForChanges(clnChat, sw_onLine, "%1 signed in", "%1 signed out") || changed
if changed then
	;playSound(getJAWSSettingsDirectory() +"\\qw_click1.wav")
endIf
endFunction

void function skypeWatcher()
let sw_tick = getTickCount()  ; makes SkypeWatch.jsl::IsSkypeWatchLoaded() work.
skypeWatchHelper()
scheduleFunction("skypeWatcher", 50)
endFunction
